#
# Immigration Mod
# CvImmigrationEventManager
# 

from CvPythonExtensions import *

import Popup as PyPopup

import CvUtil

import CvEventManager
import sys
import PyHelpers
import ImmigrationUtils
		

gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()
localText = CyTranslator()
immigrationUtils = ImmigrationUtils.ImmigrationUtils()


# Set to True to print out debug messages in the logs
g_bDebug = True

LOG_ONLY = 1
SCREEN_ONLY = 2
LOG_AND_SCREEN = 3

# Change to redirect the debug messages to the logs, screen or both.
g_iDebugMessageTarget = LOG_ONLY

			
class CvImmigrationEventManager:
	
	def __init__(self, eventManager):
		
		eventManager.addEventHandler("cityDoTurn", self.onCityDoTurn)
		eventManager.addEventHandler("mouseEvent", self.onMouseEvent)
		eventManager.addEventHandler("windowActivation", self.onWindowActivation)
		eventManager.addEventHandler("kbdEvent", self.onKbdEvent)
		eventManager.addEventHandler("unitMove", self.onUnitMove)
		eventManager.addEventHandler("cityAcquired", self.onCityAcquired)
		eventManager.addEventHandler("Update", self.onUpdate)
		eventManager.addEventHandler("EndPlayerTurn", self.onEndPlayerTurn)


	def onEndPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList
		objPlayer = gc.getPlayer(iPlayer)
		
		for i in range(objPlayer.getNumUnits()):
			objUnit = objPlayer.getUnit(i)
			
			if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_IMMIGRANT")):
				continue
			
			immigrationUtils.retargetImmigrant(objUnit)


	def onUpdate(self, argsList):
		objUnit = CyInterface().getHeadSelectedUnit()
		
		# Return immediately if there is no unit selected
		if(objUnit == None):
			return

		# Return immediately if there is no unit selected
		if(objUnit.isNone()):
			return
			
		# Return if the unit isn't an immigrant			
		if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_IMMIGRANT")):
			return
	
	
		if(objUnit.getGroup().getLengthMissionQueue() == 0):
			immigrationUtils.retargetImmigrant(objUnit)

		CyInterface().clearSelectionList()
		
	
	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
				
		objPlot = CyMap().plot(px, py)
		
		if(objPlot == None):
			return			
		#CyInterface().addImmediateMessage("%s" %(CyMap().calculatePathDistance(gc.getPlayer(0).getUnit(1).plot(),objPlot)),"")
		#if(not objPlot.isCity()):
		#	return
			
		#objCity = objPlot.getPlotCity()
		
		#CyInterface().addImmediateMessage("Foobar %s %s" %(ImmigrationUtils.g_bImmigrantsCarryReligion, ImmigrationUtils.g_iChanceImmigrantsCarryReligion),"")
		#self.generateImmigrant(objCity, false)


	def onCityAcquired(self, argsList):
		'City Acquired'
		owner,playerType,objCity,bConquest,bTrade = argsList
	
		immigrationUtils.generateImmigrant(objCity, true)


	def onUnitMove(self, argsList):
		'unit move'
		objPlot, objUnit = argsList

		immigrationUtils.handleUnitMove(objUnit, objPlot)

	
	def onKbdEvent(self, argsList):
		eventType,key,mx,my,px,py = argsList
	
		objUnit = CyInterface().getHeadSelectedUnit()
		
		# Return immediately if there is no unit selected
		if(objUnit == None):
			return

		# Return immediately if there is no unit selected
		if(objUnit.isNone()):
			return
			
		# Return if the unit isn't an immigrant			
		if(objUnit.getUnitType() != gc.getInfoTypeForString("UNIT_IMMIGRANT")):
			return
			
		if(key in (int(InputTypes.KB_ESCAPE), int(InputTypes.KB_RETURN), int(InputTypes.KB_SPACE))):
			return 
		
		# Consume the keyboard event if the currently selected unit is an 
		# immigrant
		return 1			

						
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]
		
		if(bActive):
			ImmigrationUtils.loadConfigurationData()

				
		
	def onCityDoTurn(self, argsList):
		'City Production'
		objCity = argsList[0]
		iPlayer = argsList[1]
				
		immigrationUtils.generateImmigrant(objCity, false)
